@@ -167,7 +167,7 @@ class ExprList(ExprPrint, list, ExprContainer, ExprMisc):
     # in preference to list's one.  Alternatively, add explicit
     # _repr_pretty_ method here.
     
-    def __init__(self, iterable, evalf=False):
+    def __init__(self, iterable=[], evalf=False):
 
         eiterable = []
         for item in iterable:
@@ -377,11 +377,17 @@ class Expr(ExprPrint, ExprMisc):
 
         return self.evalf()
 
-    def evalf(self):
-        """Return floating point value of expression if it can be evaluated,
-        otherwise the expression."""
+    def evalf(self, n=15, *args, **kwargs):
+        """Convert constants in an expression to floats, evaluated to `n`
+        decimal places.  If the expression is a constant, return the
+        floating point result.
 
-        return self.expr.evalf()    
+        See sympy.evalf for more details.
+
+        """
+
+        val = self.expr.evalf(n, *args, **kwargs)
+        return self.__class__(val, **self.assumptions)    
 
     def __hash__(self):
         # This is needed for Python3 so can create a dict key,
@@ -426,11 +432,12 @@ class Expr(ExprPrint, ExprMisc):
         # FIXME.  This propagates the assumptions.  There is a
         # possibility that the operation may violate them.
 
+
         # If it is not callable, directly wrap it.
         if not hasattr(a, '__call__'):
             if not isinstance(a, sym.Expr):
                 return a
-            ret = a()                
+            ret = a
             if hasattr(self, 'assumptions'):
                 return self.__class__(ret, **self.assumptions)
             return self._to_class(self.__class__, ret)
@@ -595,15 +602,24 @@ class Expr(ExprPrint, ExprMisc):
     def __rtruediv__(self, x):
         """Reverse true divide"""
 
+        from .matrix import Matrix
+        
+        if isinstance(x, Matrix):
+            return x / self.expr
+
         cls, self, x, assumptions = self.__compat_mul__(x, '/')
         return cls(x.expr / self.expr, **assumptions)
 
     def __mul__(self, x):
         """Multiply"""
         from .super import Super
+        from .matrix import Matrix
 
+        # Could return NotImplemented to trigger __rmul__ of x.
         if isinstance(x, Super):
             return x.__mul__(self)
+        elif isinstance(x, Matrix):
+            return x * self.expr
 
         cls, self, x, assumptions = self.__compat_mul__(x, '*')
         return cls(self.expr * x.expr, **assumptions)
@@ -629,12 +645,17 @@ class Expr(ExprPrint, ExprMisc):
     def __add__(self, x):
         """Add"""
 
+        from .matrix import Matrix
+
         # Convert Vs + Vt -> Voltage, etc.
         if (hasattr(self, 'superkind') and hasattr(x, 'superkind') and
             self.__class__ != x.__class__ and self.superkind ==
             x.superkind):
             cls = {'Voltage' : Voltage, 'Current' : Current}[self.superkind]
             return cls(self) + cls(x)
+
+        elif isinstance(x, Matrix):
+            return x + self.expr
         
         cls, self, x, assumptions = self.__compat_add__(x, '+')
         return cls(self.expr + x.expr, **assumptions)
@@ -1144,11 +1165,11 @@ class Expr(ExprPrint, ExprMisc):
             symbols = list(expr.free_symbols)
             if arg is None:
                 if len(symbols) == 0:
-                    return expr.evalf()                    
+                    return expr.evalf()
                 raise ValueError('Undefined symbols %s in expression %s' % (tuple(symbols), self))                                    
             if len(symbols) == 0:
                 print('Ignoring arg %s' % arg)
-                return expr.evalf()                
+                return expr.evalf()
             elif len(symbols) == 1:            
                 return evaluate_expr(expr, symbols[0], arg)
             else:
@@ -1250,6 +1271,14 @@ class Expr(ExprPrint, ExprMisc):
                 cls = class_map[(self.__class__, new.__class__)]
 
         old = symbol_map(old)
+
+        if isinstance(expr, list):
+            # Get lists from solve.  These stymy sympy's subs.
+            if len(expr) == 1:
+                expr = expr[0]
+            else:
+                print('Warning, substituting a list...')
+        
         result = self.expr.subs(old, expr)
 
         # If get empty Piecewise, then result unknowable.  TODO: sympy
@@ -1296,7 +1325,9 @@ class Expr(ExprPrint, ExprMisc):
         return call(self, arg, **assumptions)
 
     def limit(self, var, value, dir='+'):
-        """Determine limit of expression(var) at var = value."""
+        """Determine limit of expression(var) at var = value.
+        If `dir == '+'` search from right else if `dir == '-'`
+        search from left."""
 
         # Need to use lcapy sympify otherwise could use
         # getattr to call sym.limit.
@@ -1311,7 +1342,7 @@ class Expr(ExprPrint, ExprMisc):
             return self
         var = symbols[symbolnames.index(str(var))]
         
-        ret = sym.limit(self.expr, var, value)
+        ret = sym.limit(self.expr, var, value, dir=dir)
         return self.__class__(ret, **self.assumptions)
 
     def simplify(self):
@@ -1592,7 +1623,7 @@ class Expr(ExprPrint, ExprMisc):
         return self.__class__(self._ratfun.standard(), **self.assumptions)
 
     def mixedfrac(self):
-        """This is an alias for standard and my be deprecated."""
+        """This is an alias for standard and may be deprecated."""
         
         return self.standard()
 
@@ -1601,17 +1632,29 @@ class Expr(ExprPrint, ExprMisc):
 
         5 * (s**2 + 2 * s + 1) / (4 * (s**2 / 4 + 1))
 
-        See also canonical, general, standard, partfrac and ZPK."""
+        See also timeconst_terms, canonical, general, standard,
+        partfrac and ZPK."""
 
         if self._ratfun is None:
             return self.copy()        
         return self.__class__(self._ratfun.timeconst(), **self.assumptions)
 
+    def timeconst_terms(self):
+        """Convert each term of expression into time constant form."""
+
+        result = 0
+        for term in self.expr.as_ordered_terms():
+            result += self.__class__(term).timeconst()
+        return self.__class__(result, **self.assumptions)            
+
     def ZPK(self):
         """Convert to zero-pole-gain (ZPK) form (factored form).  For example,
 
         5 * (s + 1)**2 / ((s - 2 * j) * (s + 2 * j))
 
+        Note, both the numerator and denominator are expressed as
+        products of monic factors, i.e., (s + 1 / 3) rather than (3 * s + 1).
+
         See also canonical, general, standard, partfrac, and timeconst.
 
         """
@@ -1655,7 +1698,7 @@ class Expr(ExprPrint, ExprMisc):
         If norm is True, normalise coefficients to highest power is 1."""
 
         if self._ratfun is None:
-            return expr([])
+            return expr([self])
         
         try:
             z = sym.Poly(self.expr, self.var)
@@ -1860,6 +1903,137 @@ class Expr(ExprPrint, ExprMisc):
 
         result = as_sum(self.expr, self.var)
         return self.__class__(result, **self.assumptions)
+
+    def as_monic_terms(self):
+        """Rewrite terms so that each denominator is monic.
+
+        This does not expand the expression first; use `.expand()`."""
+
+        result = 0
+        for term in self.expr.as_ordered_terms():
+            N, D = as_N_D(term, self.var, monic_denominator=True)
+            result += N / D
+        return self.__class__(result, **self.assumptions)
+
+    def as_nonmonic_terms(self):
+        """Rewrite terms so that each denominator is not monic.
+
+        This does not expand the expression first; use `.expand()`."""
+
+        result = 0
+        for term in self.expr.as_ordered_terms():
+            N, D = as_N_D(term, self.var, monic_denominator=False)
+            result += N / D
+        return self.__class__(result, **self.assumptions)                
+
+    def continued_fraction_coeffs(self):
+
+        coeffs = []
+        var = self.var        
+        
+        def foo(Npoly, Dpoly):
+
+            # This seems rather complicated to extract the leading terms.
+            NLM, NLC = Npoly.LT()
+            DLM, DLC = Dpoly.LT()
+            NLT = sym.Poly(NLM.as_expr() * NLC, var)
+            DLT = sym.Poly(DLM.as_expr() * DLC, var)
+
+            Q = NLT / DLT
+            coeffs.append(Q)
+
+            Npoly2 = sym.Poly(Npoly.as_expr() - Q * Dpoly.as_expr(), var)            
+            if Npoly2 != 0:
+                foo(Dpoly, Npoly2)
+
+        N, D = self.expr.as_numer_denom()
+        Npoly = sym.Poly(N, var)
+        Dpoly = sym.Poly(D, var)
+
+        if Dpoly.degree() > Npoly.degree():
+            coeffs.append(0)
+            Npoly, Dpoly = Dpoly, Npoly
+        
+        foo(Npoly, Dpoly)
+
+        return expr(coeffs)
+    
+    def as_continued_fraction(self):
+        """Convert expression into acontinued fraction."""
+
+        def foo(coeffs):
+
+            if len(coeffs) == 1:
+                return coeffs[0]
+            return coeffs[0] + 1 / foo(coeffs[1:])
+
+        coeffs = self.continued_fraction_coeffs()
+        result = foo(coeffs)
+        return self.__class__(result, **self.assumptions)
+
+    def continued_fraction_inverse_coeffs(self):
+
+        coeffs = []
+        var = self.var
+        
+        def foo(Npoly, Dpoly):
+
+            # This seems rather complicated to extract the last non-zero terms.
+            NEM, NEC = Npoly.ET()
+            DEM, DEC = Dpoly.ET()
+            NET = NEM.as_expr() * NEC
+            DET = DEM.as_expr() * DEC
+            
+            if sym.Poly(NET, var).degree() > sym.Poly(DET, var).degree():
+                coeffs.append(0)
+                foo(Dpoly, Npoly)
+                return
+
+            Q = NET / DET
+            coeffs.append(Q)
+            
+            Npoly2 = sym.Poly(Npoly.as_expr() - Q * Dpoly.as_expr(), var)
+            if Npoly2 != 0:
+                foo(Dpoly, Npoly2)
+
+        N, D = self.expr.as_numer_denom()
+        Npoly = sym.Poly(N, var)
+        Dpoly = sym.Poly(D, var)
+
+        foo(Npoly, Dpoly)
+        return expr(coeffs)
+
+    def as_continued_fraction_inverse(self):
+
+        def foo(coeffs):
+
+            if len(coeffs) == 1:
+                return coeffs[0]
+            return coeffs[0] + 1 / foo(coeffs[1:])
+
+        coeffs = self.continued_fraction_inverse_coeffs()
+        result = foo(coeffs)
+        return self.__class__(result, **self.assumptions)
+
+
+def exprcontainer(arg, **assumptions):
+
+    if isinstance(arg, (ExprList, ExprTuple, ExprDict)):
+        return arg
+    elif isinstance(arg, list):
+        return ExprList(arg)
+    elif isinstance(arg, tuple):
+        return ExprTuple(arg)
+    elif isinstance(arg, dict):
+        return ExprDict(arg)    
+    elif isinstance(arg, np.ndarray):
+        from .vector import Vector
+        if arg.ndim > 1:
+            raise ValueError('Multidimensional arrays unsupported; convert to Matrix')
+        return Vector(arg)
+    
+    raise ValueError('Unsupported exprcontainer %s' % arg.__class__.name)
+
     
 def expr(arg, **assumptions):
     """Create Lcapy expression from arg.
@@ -1883,15 +2057,9 @@ def expr(arg, **assumptions):
     if isinstance(arg, Expr) and assumptions == {}:
         return arg
     
-    if isinstance(arg, (ExprList, ExprTuple, ExprDict)):
-        return arg
-    elif isinstance(arg, list):
-        return ExprList(arg)
-    elif isinstance(arg, tuple):
-        return ExprTuple(arg)
-    elif isinstance(arg, dict):
-        return ExprDict(arg)    
-
+    if not isinstance(arg, str) and hasattr(arg, '__iter__'):
+        return exprcontainer(arg)
+    
     expr = sympify(arg, **assumptions)
 
     symbols = expr.free_symbols
@@ -1912,12 +2080,23 @@ def symbol(name, **assumptions):
     """Create an Lcapy symbol.
 
     By default, symbols are assumed to be positive unless real is
-    defined or positive is defined as False.
-
-    """
+    defined or positive is defined as False."""
     return Expr(symsymbol(name, **assumptions))
 
 
+def symbols(names, **assumptions):
+    """Create Lcapy symbols from whitespace or comma delimiter string of
+    symbol names.  See also symbol."""
+
+    from .parser import split
+
+    namelist = split(names, ", ")
+    symbols = []
+    for name in namelist:
+        symbols.append(symbol(name, **assumptions))
+    return symbols
+
+
 from .cexpr import cExpr        
 from .fexpr import Hf, If, Vf, Yf, Zf, fExpr, fexpr
 from .sexpr import Hs, Is, Vs, Ys, Zs, sExpr, sexpr
@@ -1942,4 +2121,3 @@ try:
         formatter.type_printers[cls] = Expr._repr_latex_
 except:
     pass
-        
