@@ -1,7 +1,7 @@
 """This module provides the Immittance class, the base class for
 Admittance and Impedance and the rest of the menagerie.
 
-Copyright 2019 Michael Hayes, UCECE
+Copyright 2019-2020 Michael Hayes, UCECE
 
 """
 
@@ -14,10 +14,22 @@ from .sym import omegasym
 
 class Immitance(sExpr):
     
-    def __init__(self, val, kind=None, causal=True, **assumptions):
+    def __init__(self, val, kind=None, causal=True, positive=False, **assumptions):
+        """Create an immittance (impedance/admittance).
 
-        val = expr(val)
-        if isinstance(val, omegaExpr) and kind is None:
+        Note, by default, `positive` is False.  Thus if `val` is a
+        string, any symbols specified in the string will be assumed to
+        be complex.  For example, `Impedance('Z')`.  However, the
+        symbols R and L in `Impedance('R + s * L')` will be assumed to
+        be complex unless these symbols have been previously defined
+        otherwise.  This may stymy some simplification."""
+        
+        val = expr(val, positive=positive)
+
+        if isinstance(val, sExpr) and kind is None:
+            kind = 's'
+        elif isinstance(val, omegaExpr) and kind is None:
+            kind = omegasym
             val = val.subs(omega, s / j)
         
         super(Immitance, self).__init__(val, causal=causal, **assumptions)
@@ -25,6 +37,7 @@ class Immitance(sExpr):
 
     @property
     def _pexpr(self):
+
         """Return expression for printing."""
 
         kind = self.kind
@@ -184,3 +197,4 @@ class ImmitanceMixin(object):
     def Zs(self):
         """Generalized impedance  Z(s)."""
         return Zs(self.impedance.selectexpr(s))
+
