@@ -1,11 +1,10 @@
 """This module provides the Impedance class.  This is a generalized
 impedance (s-domain) and converts to other representations.
 
-Copyright 2019 Michael Hayes, UCECE
+Copyright 2019-2020 Michael Hayes, UCECE
 
 """
-
-
+from __future__ import division
 from .symbols import s
 from .immitance import Immitance
 
@@ -22,6 +21,13 @@ class Impedance(Immitance):
 
     """
 
+    def __rtruediv__(self, x):
+        """Reverse true divide"""
+
+        # TODO: handle Voltage / Impedance -> Current etc.
+        from .admittance import Admittance
+        return Admittance(x / self.expr)
+
     @property
     def Y(self):
         return 1 / self
@@ -39,7 +45,7 @@ class Impedance(Immitance):
         return self.jomega
 
     def cpt(self):
-        """Create oneport component."""
+        """Create oneport component.  See also network."""
         
         from .oneport import R, C, L, Z
 
@@ -57,5 +63,12 @@ class Impedance(Immitance):
             return L(z.expr)
 
         return Z(self)
-    
 
+    def network(self, form='default'):
+        """Synthesise a network with an equivalent impedance.
+        `form` includes: cauerI, cauerII, fosterI, fosterII.
+
+        Note some methods generate networks with negative value
+        components."""
+        
+        return self(s).network(form)
