@@ -2,7 +2,7 @@
 This module defines the components for modified nodal analysis.  The components
 are defined at the bottom of this file.
 
-Copyright 2015--2019 Michael Hayes, UCECE
+Copyright 2015--2020 Michael Hayes, UCECE
 
 """
 
@@ -39,7 +39,10 @@ class Cpt(ImmitanceMixin):
     need_extra_branch_current = False    
     need_control_current = False
     directive = False
-    flip_branch_current = False    
+    flip_branch_current = False
+    ignore = False
+    equipotential_nodes = ()
+    
 
     def __init__(self, cct, namespace, defname, name, cpt_type, cpt_id, string,
                  opts_string, nodes, keyword, *args):
@@ -51,7 +54,7 @@ class Cpt(ImmitanceMixin):
         self.name = name
         self.relname = name
         self.namespace = ''
-        self.nodes = nodes
+        self.nodenames = nodes
         self.relnodes = nodes
         
         parts = name.split('.')
@@ -76,7 +79,7 @@ class Cpt(ImmitanceMixin):
         self.opts = Opts(self.opts_string)
 
         # No defined cpt
-        if self.type in ('XX', ):
+        if self.type in ('XX', 'Cable'):
             self.cpt = lcapy.oneport.Dummy()
             return
 
@@ -286,61 +289,80 @@ class Cpt(ImmitanceMixin):
         """Return True if component is a source (dependent or independent)"""
         return self.dependent_source or self.independent_source
 
+    @property
+    def is_dependent_source(self):
+        """Return True if component is a dependent source"""
+        return self.dependent_source
+
+    @property
+    def is_independent_source(self):
+        """Return True if component is an independent source"""
+        return self.independent_source
+
+    @property
+    def _source_IV(self):
+
+        if self.cpt.voltage_source:
+            return self.cpt.Voc
+        elif self.cpt.current_source:
+            return self.cpt.Isc
+        else:
+            raise ValueError('%s is not a source' % self)        
+
     @property
     def is_causal(self):
         """Return True if causal component or if source produces
         a causal signal."""
 
-        if self.cpt.voltage_source:
-            return self.cpt.Voc.is_causal
-        elif self.cpt.current_source:
-            return self.cpt.Isc.is_causal
-        else:
-            raise ValueError('%s is not a source' % self)
+        return self._source_IV.is_causal
 
     @property
     def is_dc(self):
         """Return True if source is dc."""
-        
-        if self.cpt.voltage_source:
-            return self.cpt.Voc.is_dc
-        elif self.cpt.current_source:
-            return self.cpt.Isc.is_dc
-        else:
-            raise ValueError('%s is not a source' % self)
+
+        return self._source_IV.is_dc
 
     @property
     def is_ac(self):
         """Return True if source is ac."""
 
-        if self.cpt.voltage_source:
-            return self.cpt.Voc.is_ac
-        elif self.cpt.current_source:
-            return self.cpt.Isc.is_ac
-        else:
-            raise ValueError('%s is not a source' % self)
+        return self._source_IV.is_ac
 
     @property
-    def has_s(self):
-        """Return True if source has s-domain component."""
+    def has_ac(self):
+        """Return True if source has ac component."""
 
-        if self.cpt.voltage_source:
-            return self.cpt.Voc.has_s
-        elif self.cpt.current_source:
-            return self.cpt.Isc.has_s
-        else:
-            raise ValueError('%s is not a source' % self)
+        return self._source_IV.has_ac
+
+    @property
+    def has_dc(self):
+        """Return True if source has dc component."""
+
+        return self._source_IV.has_dc    
+
+    @property
+    def has_noisy(self):
+        """Return True if source has noisy component."""
+
+        return self._source_IV.has_noisy
+    
+    @property
+    def has_s_transient(self):
+        """Return True if source has transient component defined in s-domain."""
+
+        return self._source_IV.has_s_transient
+
+    @property
+    def has_t_transient(self):
+        """Return True if source has transient component defined in time domain."""
+
+        return self._source_IV.has_t_transient        
 
     @property
     def has_transient(self):
         """Return True if source has a transient component."""
 
-        if self.cpt.voltage_source:
-            return self.cpt.Voc.has_transient
-        elif self.cpt.current_source:
-            return self.cpt.Isc.has_transient
-        else:
-            raise ValueError('%s is not a source' % self)                
+        return self._source_IV.has_transient        
         
     @property
     def is_noisy(self):
@@ -353,11 +375,31 @@ class Cpt(ImmitanceMixin):
         else:
             raise ValueError('%s is not a source' % self)        
 
+    @property
+    def is_inductor(self):
+        """Return True if component is an inductor."""
+        return self.cpt.inductor
+
+    @property
+    def is_capacitor(self):
+        """Return True if component is a capacitor."""
+        return self.cpt.capacitor
+
+    @property
+    def is_reactance(self):
+        """Return True if component is a capacitor or inductor."""
+        return self.is_capacitor or self.is_inductor
+
+    @property
+    def is_resistor(self):
+        """Return True if component is a resistor."""
+        return self.cpt.resistor        
+
     @property
     def is_voltage_source(self):
         """Return True if component is a voltage source (dependent or
         independent)"""
-        return self.cpt.voltage_source
+        return self.cpt.voltage_source    
 
     @property
     def is_current_source(self):
@@ -372,10 +414,10 @@ class Cpt(ImmitanceMixin):
         return self.cpt.zeroic
 
     @property
-    def hasic(self):
+    def has_ic(self):
         """Return True if initial conditions are specified."""
 
-        return self.cpt.hasic
+        return self.cpt.has_ic
 
     @property
     def I(self):
@@ -397,13 +439,13 @@ class Cpt(ImmitanceMixin):
     def V(self):
         """Voltage drop across component."""
 
-        return self.cct.get_Vd(self.nodes[0], self.nodes[1])
+        return self.cct.get_Vd(self.nodenames[0], self.nodenames[1])
 
     @property
     def v(self):
         """Time-domain voltage drop across component."""
 
-        return self.cct.get_vd(self.nodes[0], self.nodes[1])
+        return self.cct.get_vd(self.nodenames[0], self.nodenames[1])
 
     @property
     def Isc(self):
@@ -518,7 +560,7 @@ class Cpt(ImmitanceMixin):
     @property
     def node_indexes(self):
 
-        return [self.cct._node_index(n) for n in self.nodes]
+        return [self.cct._node_index(n) for n in self.nodenames]
 
     @property
     def branch_index(self):
@@ -532,19 +574,46 @@ class Cpt(ImmitanceMixin):
     def oneport(self):
         """Create oneport object."""
         
-        return self.cct.oneport(self.nodes[1], self.nodes[0])
+        return self.cct.oneport(self.nodenames[1], self.nodenames[0])
 
     def thevenin(self):
         """Create Thevenin oneport object."""
         
-        return self.cct.thevenin(self.nodes[1], self.nodes[0])
+        return self.cct.thevenin(self.nodenames[1], self.nodenames[0])
 
     def norton(self):
         """Create Norton oneport object."""
         
-        return self.cct.norton(self.nodes[1], self.nodes[0])    
-    
+        return self.cct.norton(self.nodenames[1], self.nodenames[0])
 
+    @property
+    def nodes(self):
+        """Return list of nodes for this component.   See also
+        nodenames."""
+
+        return [self.cct.nodes[nodename] for nodename in self.nodenames]
+    
+    def connected(self):
+        """Return list of components connected to this component."""
+
+        cpts = set()
+        for node in self.nodes:
+            cpts = cpts.union(set(node.connected))
+        
+        return list(cpts)
+
+    def is_connected(self, cpt):
+        """Return True if cpt is connected to this component."""
+
+        if isinstance(cpt, str):
+            for cpt1 in self.connected:
+                if cpt1.name == cpt:
+                    return True
+            return False
+        
+        return cpt in self.connected
+
+    
 class Invalid(Cpt):
     
     @property
@@ -576,18 +645,28 @@ class Misc(Invalid):
         raise NotImplementedError('Cannot analyse misc component: %s' % self)
 
 
+class Ignored(Cpt):
+
+    ignore = True
+
+    def _stamp(self, cct):
+        # Could print warning
+        pass
+    
+
 class Dummy(Cpt):
 
     causal = True
     dc = False
     ac = False
     zeroic = True
-    hasic = None
+    has_ic = None
     noisy = False
 
 
 class XX(Dummy):
     directive = True
+    ignore = True
     
     def _stamp(self, cct):
         pass
@@ -630,29 +709,6 @@ class DependentSource(Dummy):
     
 class RLC(Cpt):
 
-    def _r_model(self):
-
-        dummy_node = self.dummy_node()
-        opts = self.opts.copy()        
-        # Strip voltage and current labels.  FIXME
-        opts = opts.strip_all_labels()
-
-        # Use Thevenin model.  This will require the current through
-        # the voltage source to be explicitly computed.
-        
-        Req = 'R_%seq' % self.type
-        Veq = 'V_%seq' % self.type        
-        
-        rnet = self._netmake_variant('R', suffix='eq',
-                                     nodes=(self.relnodes[0], dummy_node),
-                                     args=Req, opts=opts)
-
-        vnet = self._netmake_variant('V', suffix='eq',
-                                     nodes=(dummy_node, self.relnodes[1]),
-                                     args=Veq, opts=opts)
-
-        return rnet + '\n' + vnet
-
     def _s_model(self, var):
 
         if self.Voc == 0:
@@ -728,7 +784,7 @@ class RC(RLC):
         if n2 >= 0:
             cct._G[n2, n2] += Y
 
-        if cct.kind == 'ivp' and self.cpt.hasic:
+        if cct.kind == 'ivp' and self.cpt.has_ic:
             I = self.Isc.expr
             if n1 >= 0:
                 cct._Is[n1] += I 
@@ -754,10 +810,34 @@ class C(RC):
 
     def _pre_initial_model(self):
 
-        if self.cpt.v0 == 0.0:
-            return self._netmake_anon('O')
         return self._netmake_variant('V', args=self.cpt.v0)
 
+    def _r_model(self):
+
+        dummy_node = self.dummy_node()
+        opts = self.opts.copy()        
+
+        # Use Thevenin model.  This will require the current through
+        # the voltage source to be explicitly computed.
+        
+        Req = 'R%seq' % self.name
+        Veq = 'V%seq' % self.name  
+
+        opts.strip_voltage_labels()
+        rnet = self._netmake_variant('R', suffix='eq',
+                                     nodes=(self.relnodes[0], dummy_node),
+                                     args=Req, opts=opts)
+
+        opts.strip_current_labels()        
+        vnet = self._netmake_variant('V', suffix='eq',
+                                     nodes=(dummy_node, self.relnodes[1]),
+                                     args=('dc', Veq), opts=opts)
+
+        # TODO: the voltage labels should be added across an
+        # open-circuit object.
+        
+        return rnet + '\n' + vnet
+
     def _ss_model(self):
         # Perhaps mangle name to ensure it does not conflict
         # with another voltage source?
@@ -767,7 +847,7 @@ class C(RC):
     def V0(self):
         """Initial voltage (for capacitors only)."""
 
-        if self.cct.kind == 'ivp' and self.cpt.hasic:
+        if self.cct.kind == 'ivp' and self.cpt.has_ic:
             return self.cpt.v0 / s
         return 0
 
@@ -796,12 +876,19 @@ class VCVS(DependentSource):
             cct._B[n2, m] -= 1
             cct._C[m, n2] -= 1
 
-        A = cExpr(self.args[0]).expr
-        
+        Ad = cExpr(self.args[0]).expr
+        if len(self.args) > 1:
+            Ac = cExpr(self.args[1]).expr
+        else:
+            Ac = 0
+
+        Ap = (Ac / 2 + Ad)
+        Am = (Ac / 2 - Ad)        
+            
         if n3 >= 0:
-            cct._C[m, n3] -= A
+            cct._C[m, n3] -= Ap
         if n4 >= 0:
-            cct._C[m, n4] += A
+            cct._C[m, n4] -= Am
 
     def _kill(self):
         newopts = self.opts.copy()
@@ -862,6 +949,7 @@ class VCCS(DependentSource):
 
         return self._netmake_anon('O', opts=newopts)        
 
+    
 class GY(Dummy):
     """Gyrator"""    
 
@@ -966,8 +1054,7 @@ class I(IndependentSource):
 
     def _pre_initial_model(self):
 
-        # Assume IC zero.  FIXME
-        return self._netmake_anon('O', opts=newopts)
+        return self._netmake(args=self.cpt.Isc.pre_initial_value())        
 
 
 class K(Dummy):
@@ -1016,11 +1103,37 @@ class L(RLC):
     need_branch_current = True
     reactive = True
 
+    def _r_model(self):
+
+        dummy_node = self.dummy_node()
+        opts = self.opts.copy()        
+
+        # Use Thevenin model.  This will require the current through
+        # the voltage source to be explicitly computed.
+        
+        Req = 'R%seq' % self.name
+        Veq = 'V%seq' % self.name        
+
+        opts.strip_voltage_labels()
+        rnet = self._netmake_variant('R', suffix='eq',
+                                     nodes=(self.relnodes[0], dummy_node),
+                                     args=Req, opts=opts)
+
+        opts.strip_current_labels()                
+        vnet = self._netmake_variant('V', suffix='eq',
+                                     nodes=(dummy_node, self.relnodes[1]),
+                                     args=('dc', Veq), opts=opts)
+
+        # TODO: the voltage labels should be added across an
+        # open-circuit object.
+
+        return rnet + '\n' + vnet
+
     @property
     def I0(self):
         """Initial current (for capacitors only)."""
 
-        if self.cct.kind == 'ivp' and self.cpt.hasic:
+        if self.cct.kind == 'ivp' and self.cpt.has_ic:
             return self.cpt.i0 / s
         return 0
     
@@ -1057,7 +1170,7 @@ class L(RLC):
 
         cct._D[m, m] += -Z
 
-        if cct.kind == 'ivp' and self.cpt.hasic:
+        if cct.kind == 'ivp' and self.cpt.has_ic:
             V = self.Voc.expr            
             cct._Es[m] += V
 
@@ -1068,10 +1181,9 @@ class L(RLC):
     
     def _pre_initial_model(self):
 
-        if self.cpt.i0 == 0.0:
-            return self._netmake_anon('W')                    
         return self._netmake_variant('I', args=self.cpt.i0)
 
+    
 class O(Dummy):
     """Open circuit"""
 
@@ -1139,6 +1251,7 @@ class SPpm(Dummy):
         if n2 >= 0:
             cct._C[m, n2] += 1
 
+            
 class SPppp(Dummy):
 
     need_branch_current = True
@@ -1158,6 +1271,7 @@ class SPppp(Dummy):
         if n4 >= 0:
             cct._C[m, n4] -= 1
 
+            
 class SPpmm(Dummy):
 
     need_branch_current = True
@@ -1232,17 +1346,18 @@ class TFtap(Cpt):
         raise NotImplementedError('Cannot analyse tapped transformer %s' % self)
 
 
-class TL(Misc):
+class TL(Ignored):
     """Transmission line"""
 
-    # TODO
-    pass
+    equipotential_nodes = (('out1', 'in1'),
+                           ('out2', 'in2'))
+    
 
-
-class Cable(Misc):
+class Cable(Ignored):
     """Cable"""
 
-    pass
+    equipotential_nodes = (('in+', 'out+'), ('in-', 'out-'), ('in', 'out'),
+                           ('ignd', 'ognd', 'b', 't'), ('mid', 'out'))
 
 
 class TP(Misc):
@@ -1311,8 +1426,7 @@ class V(IndependentSource):
 
     def _pre_initial_model(self):
 
-        # Assume IC zero.  FIXME
-        return self._netmake_anon('W')
+        return self._netmake(args=self.cpt.Voc.pre_initial_value())
 
 
 class W(Dummy):
