@@ -17,81 +17,52 @@ from .current import Current, Iname
 from .schematic import Schematic
 from .mna import MNAMixin, Nodedict, Branchdict
 from .statespace import StateSpace
+from .simulator import Simulator
 from .netfile import NetfileMixin
 from .expr import Expr
 from .state import state
 from .attrdict import AttrDict
-from .immitance import ImmitanceMixin
 from .impedance import Impedance
 from .admittance import Admittance
+from .matrix import Matrix
+from .node import Node
 from . import mnacpts
 from copy import copy
 from collections import OrderedDict
 
 
-class Node(ImmitanceMixin):
+class EquipotentialNodes(dict):
 
-    def __init__(self, cct, name):
+    def add(self, nodenames):
+        for key in nodenames:
+            self[key] = [key]
 
-        self.cct = cct
-        self.name = name
-        self.pos = None
-        self.port = False
-        parts = name.split('_')
-        self.rootname = parts[0] if name[0] != '_' else name
-        self.primary = len(parts) == 1
-        # List of elements connected to this node.
-        self.list = []
+    def find_key(self, n):
 
-    @property
-    def V(self):
-        """Node voltage with respect to ground."""
+        for key, nodes in self.items():
+            if n in nodes:
+                return key
+        return None
 
-        return self.cct.get_Vd(self.name, '0')
+    def add_wire(self, n1, n2):
 
-    @property
-    def v(self):
-        """Node time-domain voltage with respect to ground."""
+        key1 = self.find_key(n1)
+        key2 = self.find_key(n2)
 
-        return self.cct.get_vd(self.name, '0')
+        if key1 is None or key2 is None:
+            return
 
-    @property
-    def admittance(self):
-        """Driving-point admittance between node and ground."""
+        if key1 != key2:
+            # Merge equipotential nodes.
+            self[key1].extend(self.pop(key2))
 
-        return self.cct.admittance(self.name, '0')
+    def add_wires(self, nodes):
 
-    @property
-    def impedance(self):
-        """Driving-point impedance between node and ground."""        
-
-        return self.cct.impedance(self.name, '0')        
-
-    def append(self, cpt):
-
-        if cpt.type in ('P', ):
-            self.port = True
-
-        self.list.append(cpt)
-
-    def oneport(self, node=0):
-        """Create oneport object with respect to specified node
-        (default ground)."""
-
-        return self.cct.oneport(self.name, node)        
-        
-    def thevenin(self, node=0):
-        """Create Thevenin oneport object with respect to specified
-        node (default ground)."""
-        
-        return self.cct.thevenin(self.name, node)
-
-    def norton(self, node=0):
-        """Create Norton oneport object with respect to specified node
-        (default ground)."""
-        
-        return self.cct.norton(self.name, node)    
-        
+        # Iterate over all pairs.
+        for i in range(len(nodes)):
+            for j in range(i + 1, len(nodes)):
+                self.add_wire(nodes[i], nodes[j])
+            
 
 class NetlistNamespace(object):
     """This class allows elements, nodes, or other namespaces
@@ -152,6 +123,12 @@ class NetlistNamespace(object):
 
         return self.__getitem__(attr)
 
+    @property
+    def components(self):
+        """Return ordered dictionary of components."""
+
+        return self.elements
+
     def netlist(self):
         """Return the current netlist for this namespace."""
 
@@ -330,7 +307,7 @@ class NetlistMixin(object):
 
         self._elements[cpt.name] = cpt
 
-        for node in cpt.nodes:
+        for node in cpt.nodenames:
             self._node_add(node, cpt)
 
         self._namespace_add(cpt.namespace)
@@ -372,7 +349,12 @@ class NetlistMixin(object):
         return Netlist(context=context)
 
     def remove(self, name):
-        """Remove specified element."""
+        """Remove specified element or elements specified in list."""
+
+        if isinstance(name, (list, tuple)):
+            for name1 in name:
+                self.remove(name1)
+            return self
 
         self._invalidate()
 
@@ -381,6 +363,23 @@ class NetlistMixin(object):
         self._elements.pop(name)
         # TODO, remove nodes that are only connected
         # to this component.
+        return self
+
+    @property
+    def super_nodes(self):
+        """Super nodes are nodes linked by voltage sources."""
+
+        snodes = []
+
+        for elt in self.elements.values():
+            if not elt.is_voltage_source:
+                continue
+            snodes.append(elt.nodenames[0:2])
+
+        from .utils import merge_common
+
+        # Merge supernodes to create super-supernodes...
+        return list(merge_common(snodes))
 
     @property
     def equipotential_nodes(self):
@@ -388,27 +387,16 @@ class NetlistMixin(object):
         This returns a dictionary keyed by the unique node names with
         values being lists of nodes of the same potential."""
 
-        enodes = {}
-        for key in self.nodes.keys():
-            enodes[key] = [key]
+        enodes = EquipotentialNodes()
+        enodes.add(self.nodes.keys())
 
         # Then augment with nodes connected by wires.
         for m, elt in enumerate(self.elements.values()):
-            if elt.type not in ('W', ):
-                continue
-
-            n1, n2 = elt.nodes
-
-            for key1, nodes in enodes.items():
-                if n1 in nodes:
-                    break
-
-            for key2, nodes in enodes.items():
-                if n2 in nodes:
-                    break
-
-            if key1 != key2:
-                enodes[key1].extend(enodes.pop(key2))
+            if elt.type == 'W':
+                enodes.add_wire(*elt.nodenames)
+            else:
+                for connections in elt.equipotential_nodes:
+                    enodes.add_wires([elt.name + '.' + n for n in connections])
 
         # Alter keys to avoid underscore and to ensure that have a '0'
         # key if possible.
@@ -596,7 +584,7 @@ class NetlistMixin(object):
             if isinstance(cpt, Node):
                 Np, Nm = cpt.name, 0
             else:
-                Np, Nm = cpt.nodes[0:2]
+                Np, Nm = cpt.nodenames[0:2]
         return Np, Nm
     
     def Voc(self, Np, Nm=None):
@@ -782,12 +770,14 @@ class NetlistMixin(object):
 
         return Hs(V2.laplace() / V1.laplace(), causal=True)
 
-    def Amatrix(self, N1p, N1m, N2p, N2m):
-        """Create A matrix from network, where:
+    def Aparams(self, N1p, N1m, N2p, N2m):
+        """Create A-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
         I1 is the current flowing into N1p and out of N1m
         I2 is the current flowing into N2p and out of N2m
         V1 is V[N1p] - V[N1m]
         V2 is V[N2p] - V[N2m]
+
+        See also  Bparams, Gparams, Hparams, Sparams, Tparams, Yparams, and Zparams.
         """
 
         from .twoport import AMatrix
@@ -813,12 +803,19 @@ class NetlistMixin(object):
             net.add('I1_ %s %s {DiracDelta(t)}' % (N1p, N1m))
 
             # A21 = I1 / V2 with I2 = 0
-            # Apply I1 and measure I2 with port 2 open-circuit
-            A21 = Ys(-net.I1_.I(s) / net.Voc(N2p, N2m)(s))
+            # Apply I1 and measure V2 with port 2 open-circuit
+            try:
+                A21 = Ys(-1 / net.Voc(N2p, N2m)(s))                
+            except ValueError:
+                # It is likely there is an open-circuit.                
+                net2 = net.copy()
+                net2.add('W %s %s' % (N2p, N2m))
+                A21 = Ys(-net2.I1_.I(s) / net2.Voc(N2p, N2m)(s))
+                A21 = 0                
 
             # A22 = I1 / I2 with V2 = 0
             # Apply I1 and measure I2 with port 2 short-circuit
-            A22 = Hs(-net.I1_.I(s) / net.Isc(N2p, N2m)(s))
+            A22 = Hs(1 / net.Isc(N2p, N2m)(s))
 
             net.remove('I1_')
             A = AMatrix(((A11, A12), (A21, A22)))
@@ -827,17 +824,85 @@ class NetlistMixin(object):
         except ValueError:
             raise ValueError('Cannot create A matrix')
 
-
-    def Zmatrix(self, N1p, N1m, N2p, N2m):
-        """Create Z matrix from network, where:
+    def Bparams(self, N1p, N1m, N2p, N2m):
+        """Create B-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
         I1 is the current flowing into N1p and out of N1m
         I2 is the current flowing into N2p and out of N2m
         V1 is V[N1p] - V[N1m]
         V2 is V[N2p] - V[N2m]
-        """
 
+        See also  Aparams, Gparams, Hparams, Sparams, Tparams, Yparams, and Zparams.
+        """
+        return self.Aparams(N1p, N1m, N2p, N2m).Bparams
+
+    def Gparams(self, N1p, N1m, N2p, N2m):
+        """Create G-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+
+        See also  Aparams, Bparams, Hparams, Sparams, Tparams, Yparams, and Zparams.
+        """
+        return self.Aparams(N1p, N1m, N2p, N2m).Gparams
+
+    def Hparams(self, N1p, N1m, N2p, N2m):
+        """Create H-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+
+        See also  Aparams, Bparams, Gparams, Sparams, Tparams, Yparams, and Zparams.
+        """
+        return self.Aparams(N1p, N1m, N2p, N2m).Hparams
+
+    def Sparams(self, N1p, N1m, N2p, N2m):
+        """Create S-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+
+        See also  Aparams, Bparams, Gparams, Hparams, Tparams, Yparams, and Zparams.
+        """
+        return self.Aparams(N1p, N1m, N2p, N2m).Sparams
+
+    def Tparams(self, N1p, N1m, N2p, N2m):
+        """Create T-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+
+        See also  Aparams, Bparams, Gparams, Hparams, Sparams, Yparams, and Zparams.
+        """
+        return self.Tparams(N1p, N1m, N2p, N2m).Hparams
+
+    def Yparams(self, N1p, N1m, N2p, N2m):
+        """Create Y-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+
+        See also  Aparams, Bparams, Gparams, Hparams, Sparams, Tparams, and Zparams.
+        """
+        return self.Zparams(N1p, N1m, N2p, N2m).Yparams            
+
+    def Zparams(self, N1p, N1m, N2p, N2m):
+        """Create Z-parameters for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+
+        See also  Aparams, Bparams, Gparams, Hparams, Sparams, Tparams, and Yparams.
+        """
         from .twoport import ZMatrix
 
+        # TODO, generalise to multiports.
+        
         N1p, N1m, N2p, N2m = self._check_nodes(N1p, N1m, N2p, N2m)        
         net = self.kill()
         if '0' not in net.nodes:
@@ -848,11 +913,11 @@ class NetlistMixin(object):
 
             # Z11 = V1 / I1 with I2 = 0
             # Apply I1 and measure V1 with port 2 open-circuit
-            Z11 = Zs(net.Voc(N1p, N1m)(s) / net.I1_.I(s))
+            Z11 = Zs(net.Voc(N1p, N1m)(s))
 
             # Z21 = V2 / I1 with I2 = 0
             # Apply I1 and measure V2 with port 2 open-circuit
-            Z21 = Zs(net.Voc(N2p, N2m)(s) / net.I1_.I(s))          
+            Z21 = Zs(net.Voc(N2p, N2m)(s))          
 
             net.remove('I1_')
 
@@ -860,20 +925,128 @@ class NetlistMixin(object):
 
             # Z12 = V1 / I2 with I1 = 0
             # Apply I2 and measure V1 with port 1 open-circuit
-            Z12 = Zs(net.Voc(N1p, N1m)(s) / net.I2_.I(s))
+            Z12 = Zs(net.Voc(N1p, N1m)(s))
 
             # Z22 = V2 / I2 with I1 = 0
             # Apply I2 and measure V2 with port 1 open-circuit
-            Z22 = Zs(net.Voc(N2p, N2m)(s) / net.I2_.I(s))          
+            Z22 = Zs(net.Voc(N2p, N2m)(s))          
 
             net.remove('I2_')            
 
             Z = ZMatrix(((Z11, Z12), (Z21, Z22)))
             return Z
 
+        except ValueError:
+            raise ValueError('Cannot create Z matrix')
+
+    def Yparamsn(self, *nodes):
+        """Create Y-parameters for N-port defined by list of node-pairs.
+
+        See also Yparams for a two port.
+
+        """
+
+        nodes = self._check_nodes(*nodes)
+        if len(nodes) % 2 == 1:
+            raise ValueError('Need an even number of nodes.')
+        ports = []
+        for m in range(len(nodes) // 2):
+            ports.append((nodes[m * 2], nodes[m * 2 + 1]))
+        
+        net = self.kill()
+        if '0' not in net.nodes:
+            net.add('W %s 0' % nodes[1])
+
+        try:
+
+            Y = Matrix.zeros(len(ports))
+            
+            for col in range(len(ports)):
+
+                for row in range(len(ports)):
+                    if row == col:
+                        net.add('V%d_ %s %s {DiracDelta(t)}' % (row, ports[row][0], ports[row][1]))
+                    else:
+                        net.add('V%d_ %s %s 0' % (row, ports[row][0], ports[row][1]))                        
+
+                for row in range(len(ports)):
+                    Y[row, col] = Ys(net.elements['V%d_' % row].I(s))
+
+                for row in range(len(ports)):                        
+                    net.remove('V%d_' % row)
+            return Y
+
+        except ValueError:
+            raise ValueError('Cannot create Y matrix')
+
+    def Yparams3(self, N1p, N1m, N2p, N2m, N3p, N3m):
+        """Create Y-parameters for three-port defined by nodes N1p, N1m, N2p,
+        N2m, N3p, and N3m, where:
+
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        I3 is the current flowing into N3p and out of N3m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+        V3 is V[N3p] - V[N3m]
+
+        See also Yparams for a two port and Yparamsn for an N-port.
+
+        """
+
+        return self.Yparamsn(N1p, N1m, N2p, N2m, N3p, N3m)        
+
+    def Zparamsn(self, *nodes):
+        """Create Z-parameters for N-port defined by list of node-pairs.
+
+        See also Zparams for a two port.
+
+        """
+
+        nodes = self._check_nodes(*nodes)
+        if len(nodes) % 2 == 1:
+            raise ValueError('Need an even number of nodes.')
+        ports = []
+        for m in range(len(nodes) // 2):
+            ports.append((nodes[m * 2], nodes[m * 2 + 1]))
+        
+        net = self.kill()
+        if '0' not in net.nodes:
+            net.add('W %s 0' % nodes[1])
+
+        try:
+
+            Z = Matrix.zeros(len(ports))
+            
+            for col in range(len(ports)):
+                net.add('I_ %s %s {DiracDelta(t)}' % (ports[col][0], ports[col][1]))
+
+                for row in range(len(ports)):                
+                    Z[row, col] = Zs(net.Voc(ports[row][0], ports[row][1])(s))
+
+                net.remove('I_')
+            return Z
+
         except ValueError:
             raise ValueError('Cannot create Z matrix')        
 
+    def Zparams3(self, N1p, N1m, N2p, N2m, N3p, N3m):
+        """Create Z-parameters for three-port defined by nodes N1p, N1m, N2p,
+        N2m, N3p, and N3m, where:
+
+        I1 is the current flowing into N1p and out of N1m
+        I2 is the current flowing into N2p and out of N2m
+        I3 is the current flowing into N3p and out of N3m
+        V1 is V[N1p] - V[N1m]
+        V2 is V[N2p] - V[N2m]
+        V3 is V[N3p] - V[N3m]
+
+        See also Zparams for a two port and Zparamsn for an N-port.
+
+        """
+
+        return self.Zparamsn(N1p, N1m, N2p, N2m, N3p, N3m)
+
     def save(self, filename):
         """Save netlist to file."""
 
@@ -1012,7 +1185,7 @@ class NetlistMixin(object):
         return self._noisy(resistors)
 
     def twoport(self, N1p, N1m, N2p, N2m, model='B'):
-        """Create s-domain twoport model from network, where:
+        """Create s-domain twoport model for two-port defined by nodes N1p, N1m, N2p, and N2m, where:
         I1 is the current flowing into N1p and out of N1m
         I2 is the current flowing into N2p and out of N2m
         V1 is V[N1p] - V[N1m]
@@ -1030,22 +1203,22 @@ class NetlistMixin(object):
         if model == 'B':
             V2b = net.Voc(N2p, N2m)(s)
             I2b = net.Isc(N2p, N2m)(s)
-            A = net.Amatrix(N1p, N1m, N2p, N2m)
-            return TwoPortBModel(A.B, V2b, I2b)
+            A = net.Aparams(N1p, N1m, N2p, N2m)
+            return TwoPortBModel(A.Bparams, V2b, I2b)
         elif model == 'Z':
             V1 = net.Voc(N1p, N1m)(s)
             V2 = net.Voc(N2p, N2m)(s)
-            Z = net.Zmatrix(N1p, N1m, N2p, N2m)            
+            Z = net.Zparams(N1p, N1m, N2p, N2m)            
             return TwoPortZModel(Z, V1, V2)
         elif model == 'Z':
             I1 = net.Isc(N1p, N1m)(s)
             I2 = net.Isc(N2p, N2m)(s)
-            Z = net.Zmatrix(N1p, N1m, N2p, N2m)            
+            Z = net.Zparams(N1p, N1m, N2p, N2m)            
             return TwoPortYModel(Z.Y, I1, I2)        
         elif model == 'H':
             V1 = net.Voc(N1p, N1m)(s)
             I2 = net.Isc(N2p, N2m)(s)
-            Z = net.Zmatrix(N1p, N1m, N2p, N2m)            
+            Z = net.Zparams(N1p, N1m, N2p, N2m)            
             return TwoPortHModel(Z.H, V1, I2)
         else:
             raise ValueError('Model %s unknown, must be B, H, Y, or Z' % model)
@@ -1066,6 +1239,16 @@ class NetlistMixin(object):
         self._sch = sch
         return sch
 
+    @property
+    def sim(self):
+        """Generate simulation object."""        
+
+        if hasattr(self, '_sim'):
+            return self._sim
+
+        self._sim = Simulator(self)
+        return self._sim
+    
     @property
     def ss(self):
         """Generate state-space representation."""        
@@ -1220,6 +1403,15 @@ class NetlistMixin(object):
         initial value problem."""
         return self.analysis['ac']
 
+    @property
+    def is_superposition(self):
+        """Return True if netlist needs to be solved using multiple approaches,
+        e.g., ac and dc"""
+        
+        return ((self.has_ac and self.has_dc) or
+                (self.has_ac and self.has_transient) or
+                (self.has_dc and self.has_transient))
+
     @property
     def has_dc(self):
         """Return True if any independent source has a DC component."""
@@ -1231,8 +1423,8 @@ class NetlistMixin(object):
         return self.analysis['has_ac']
 
     @property
-    def has_s(self):
-        """Return True if any independent source has an s-domain component."""
+    def has_s_transient(self):
+        """Return True if any independent source has a transient component defined in s-domain."""
         return self.analysis['has_s']    
 
     @property    
@@ -1264,7 +1456,7 @@ class NetlistMixin(object):
         """Return components that allow initial conditions but do not have
         them explicitly defined."""
 
-        return dict((key, cpt) for key, cpt in self.elements.items() if cpt.hasic is False)
+        return dict((key, cpt) for key, cpt in self.elements.items() if cpt.has_ic is False)
 
     @property
     def sources(self):
@@ -1278,6 +1470,12 @@ class NetlistMixin(object):
         """Return dictionary of reactances."""
 
         return self.analysis['reactances']
+
+    @property
+    def ics(self):
+        """Return dictionary of components with initial conditions."""
+
+        return self.analysis['ics']    
     
     @property
     def independent_sources(self):
@@ -1330,7 +1528,7 @@ class NetlistMixin(object):
 
     @property
     def control_sources(self):
-        """Return dictionary of voltage sources required to specify control
+        """Return list of voltage sources required to specify control
         current for CCVS and CCCS components."""
         return self.analysis['control_sources']        
 
@@ -1344,7 +1542,7 @@ class NetlistMixin(object):
 
     def analyse(self):
 
-        hasic = False
+        has_ic = False
         zeroic = True
         has_s = False
         has_transient = False        
@@ -1356,18 +1554,20 @@ class NetlistMixin(object):
         dependent_sources = []        
         control_sources = []
         reactances = []
+        ics = []
         
         for key, elt in self.elements.items():
             if elt.need_control_current:
                 control_sources.append(elt.args[0])
-            if elt.hasic is not None:
-                if elt.hasic:
-                    hasic = True
+            if elt.has_ic is not None:
+                if elt.has_ic:
+                    has_ic = True
+                    ics.append(key)
                 if not elt.zeroic:
                     zeroic = False
             if elt.independent_source:
                 independent_sources.append(key)
-                if elt.has_s:
+                if elt.has_s_transient:
                     has_s = True
                 if elt.has_transient:
                     has_transient = True                    
@@ -1387,22 +1587,23 @@ class NetlistMixin(object):
                     
         analysis = {} 
         analysis['zeroic'] = zeroic
-        analysis['has_ic'] = hasic        
-        analysis['ivp'] = hasic
+        analysis['has_ic'] = has_ic        
+        analysis['ivp'] = has_ic
         analysis['has_dc'] = dc_count > 0
         analysis['has_ac'] = ac_count > 0        
         analysis['has_s'] = has_s
         analysis['has_transient'] = has_transient
-        analysis['reactances'] = reactances 
+        analysis['reactances'] = reactances
+        analysis['ics'] = ics         
         analysis['dependent_sources'] = dependent_sources        
         analysis['independent_sources'] = independent_sources
         analysis['control_sources'] = control_sources        
-        analysis['ac'] = ac_count > 0 and (num_sources == ac_count) and not hasic
-        analysis['dc'] = dc_count > 0 and (num_sources == dc_count) and not hasic
+        analysis['ac'] = ac_count > 0 and (num_sources == ac_count) and not has_ic
+        analysis['dc'] = dc_count > 0 and (num_sources == dc_count) and not has_ic
         analysis['causal'] = causal and zeroic
         analysis['time_domain'] = not reactive and not has_s
 
-        if not reactive and hasic:
+        if not reactive and has_ic:
             raise ValueError('Non-reactive component with initial conditions')
         return analysis
 
@@ -1427,8 +1628,9 @@ class NetlistMixin(object):
             return
 
         if self.is_ivp:
-            print('This has initial conditions so is an initial value problem '
-                  'solved in the s-domain using Laplace transforms.')
+            print('This has initial conditions for %s so is an initial value '
+                  'problem solved in the s-domain using Laplace transforms.' %
+                  ', '.join(self.ics))
             return
 
         if len(groups) > 1:
@@ -1442,7 +1644,7 @@ class NetlistMixin(object):
                 print(describe_analysis('DC', sources))
             elif kind == 's':
                 print(describe_analysis('Laplace', sources))
-            elif kind == 'time':
+            elif kind in ('t', 'time'):
                 print(describe_analysis('Time-domain', sources))
 
     def Vname(self, name):
@@ -1492,7 +1694,7 @@ class Netlist(NetlistMixin, NetfileMixin):
             except:
                 pass
 
-    def _sub_make(self):
+    def _groups(self):
 
         groups = self.independent_source_groups()        
             
@@ -1526,6 +1728,11 @@ class Netlist(NetlistMixin, NetfileMixin):
         else:
             groups = self.independent_source_groups(transform=True)        
         
+        return groups
+        
+    def _sub_make(self):
+
+        groups = self._groups()
         self._sub = Transformdomains()
 
         for kind, sources in groups.items():
@@ -1546,6 +1753,10 @@ class Netlist(NetlistMixin, NetfileMixin):
 
         return self._sub_make()
 
+    @property
+    def subcircuits(self):
+        return self.sub
+
     @property
     def kinds(self):
         """Return list of transform domain kinds."""
