@@ -14,6 +14,9 @@ class Network(object):
 
     voltage_source = False
     current_source = False
+    inductor = False
+    capacitor = False
+    resistor = False
 
     # True if initial conditions are zero (or unspecified).
     zeroic = True
@@ -21,19 +24,11 @@ class Network(object):
     # None if component does not have initial conditions.
     # True if initial conditions are specified.
     # False if initial conditions are not specified.
-    hasic = None
+    has_ic = None
 
     netname = ''
     netkeyword = ''
 
-    def _make_id(self, kind):
-        """Make identifier"""
-
-        if kind not in self._anon:
-            self._anon[kind] = 0
-        self._anon[kind] += 1
-        return self._anon[kind]
-        
     def _tweak_args(self):
 
         if not hasattr(self, 'args'):
@@ -73,6 +68,10 @@ class Network(object):
         argsrepr = ', '.join([latex(arg, **kwargs) for arg in self._tweak_args()])
         return '\\mathrm{%s}(%s)' % (self.__class__.__name__, argsrepr)
 
+    def pprint(self):
+
+        print(self.pretty())
+    
     @property
     def analysis(self):
         return self.cct.analysis
@@ -92,54 +91,57 @@ class Network(object):
             self._add(net)
 
         # Hack, create ground reference.
-        self._add('W %d 0' % (self.node - 1))
+        self._add('W %d 0' % (self._node - 1))
+
+    def _net_make(self, netlist, n1=None, n2=None, dir='right'):
+
+        net = self
+        if n2 == None:
+            n2 = net._node
+        if n1 == None:
+            n1 = net._node
+        
+        netname = net.__class__.__name__ if net.netname == '' else net.netname
+
+        netid = netlist._make_id(netname)
+        if net.netkeyword != '':
+            return '%s%s %s %s %s %s; %s' % (netname, netid,
+                                             n1, n2, 
+                                             net.netkeyword,
+                                             netlist._netargs(net), dir)
+        else:
+            return '%s%s %s %s %s; %s' % (netname, netid,
+                                          n1, n2, netlist._netargs(net), dir)
 
     @property 
-    def node(self):
+    def _depths(self):
+        return [net._depth for net in self.args]
+        
+    @property 
+    def _depth(self):
+        from .oneport import Ser, Par
+        
+        if not isinstance(self, (Ser, Par)):
+            return 0
+        
+        depths = self._depths
+        return 1 + max(depths)
+        
+    def netlist(self, form='horizontal', evalf=None):
+        """Create a netlist.
 
-        if not hasattr(self, 'node_counter'):
-            self.node_counter = 0
-        ret = self.node_counter
-        self.node_counter += 1
-        return ret
+        `form` can be 'horizontal', 'vertical', or 'ladder'.
 
-    def netargs(self):
+        `evalf` can be False or an integer specifying the number of
+        decimal places used to evaluate floats.
+        """
 
-        def quote(arg):
+        if form == 'ladder':
+            from .laddermaker import LadderMaker
+            return LadderMaker(self, form=form, evalf=evalf)()
 
-            # TODO: make more robust to catch expressions.
-            if ('(' in arg) or (')' in arg) or (' ' in arg) or (',' in arg) or ('*' in arg) or ('/' in arg):
-                return '{%s}' % arg
-            return arg
-
-        return ' '.join([quote(str(arg)) for arg in self.args])
-
-    def net_make(self, net, n1=None, n2=None):
-
-        if n1 == None:
-            n1 = net.node
-        if n2 == None:
-            n2 = net.node
-
-        netname = self.__class__.__name__ if self.netname == '' else self.netname
-
-        netid = net._make_id(netname)
-        if self.netkeyword != '':
-            return '%s%s %s %s %s %s; right' % (netname, netid,
-                                                n1, n2, 
-                                                self.netkeyword, self.netargs())
-        else:
-            return '%s%s %s %s %s; right' % (netname, netid,
-                                             n1, n2, self.netargs())
-
-    def netlist(self):
-
-        # Enumerate from node 0
-        self.node_counter = 0
-        self._anon = {}
-        n1 = self.node
-        n2 = self.node        
-        return self.net_make(self, n2, n1)
+        from .netlistmaker import NetlistMaker        
+        return NetlistMaker(self, form=form, evalf=evalf)()
 
     def pdb(self):
         """Enter the python debugger."""
@@ -147,21 +149,17 @@ class Network(object):
         import pdb; pdb.set_trace()
         return self
     
-    @property
-    def sch(self):
+    def sch(self, form, evalf=False):
         """Convert a Network object into a Schematic object."""
 
-        if hasattr(self, '_sch'):
-            return self._sch
-
-        netlist = self.netlist()
+        netlist = self.netlist(form=form, evalf=evalf)
         sch = Schematic()
         for net in netlist.split('\n'):
             sch.add(net)
-        self._sch = sch
+
         return sch
 
-    def draw(self, filename=None, **kwargs):
+    def draw(self, filename=None, form='horizontal', evalf=False, **kwargs):
         """Draw schematic of network.
 
         filename specifies the name of the file to produce.  If None,
@@ -169,7 +167,12 @@ class Network(object):
 
         Note, if using Jupyter, then need to first issue command %matplotlib inline
 
-        kwargs include:
+        `form` is either 'horizontal', 'vertical', or 'ladder'.
+
+        `evalf` can be False or an integer specifying the number of
+        decimal places used to evaluate floats.
+
+        `kwargs` include:
            label_ids: True to show component ids
            label_values: True to display component values
            draw_nodes: True to show all nodes, False to show no nodes, 
@@ -199,7 +202,7 @@ class Network(object):
         if 'draw_nodes' not in kwargs:
             kwargs['draw_nodes'] = 'connections'
         
-        self.sch.draw(filename=filename, **kwargs)
+        self.sch(form=form, evalf=evalf).draw(filename=filename, **kwargs)
         
     @property
     def cct(self):
@@ -271,3 +274,14 @@ class Network(object):
         symbols = self.symbols
         symbols.update(state.global_context.symbols)
         return symbols
+
+    def transform(self, form='causerI'):
+        """Transform the network into an alternative form.  The transformation
+        is performed using network synthesis of the network's
+        impedance (note, this ignores the sources).  `form` includes:
+        cauerI, cauerII, fosterI, fosterII.
+
+        Note some methods generate networks with negative value
+        components."""
+
+        return self.Z(s).network(form)
