@@ -40,6 +40,8 @@ class noiseExpr(Expr):
 
     """
     one_sided = True
+    is_real = True
+    is_positive = True
 
     def _new_nid(self):
         state.context.nid += 1
@@ -58,7 +60,6 @@ class noiseExpr(Expr):
     def nid(self):
         return self.assumptions['nid']
 
-
     def subs(self, *args, **kwargs):
         return super(noiseExpr, self).subs(*args, nid=self.nid, **kwargs)
 
@@ -108,7 +109,7 @@ class noiseExpr(Expr):
         return self + x
 
     def __rsub__(self, x):
-        raise ValueError('Cannot subtract %s and %s' % (self, x))                
+        raise ValueError('Cannot subtract %s and %s' % (self, x))          
 
     def __mul__(self, x):
         if isinstance(x, noiseExpr) and self.nid != x.nid:
