@@ -1,9 +1,18 @@
+"""This module provides the noisefExpr class to represent
+f-domain (Fourier domain) noise expressions.
+
+Copyright 2014--2019 Michael Hayes, UCECE
+
+"""
 from __future__ import division
 from .sym import symsimplify
 from .functions import sqrt
 from .sym import pi, omegasym, fsym
 from .state import state
+from .expr import expr
 from .noiseexpr import noiseExpr
+from .omegaexpr import omega, omegaExpr
+from .fexpr import fExpr
 import sympy as sym
 import numpy as np
 
@@ -45,30 +54,6 @@ class noisefExpr(noiseExpr):
     domain_name = 'Frequency'
     domain_units = 'Hz'    
 
-    def __call__(self, arg, **assumptions):
-        """Transform domain or substitute arg for variable. 
-        
-        Substitution is performed if arg is a tuple, list, numpy
-        array, or constant.  If arg is a tuple or list return a list.
-        If arg is an numpy array, return numpy array.
-
-        Domain transformation is performed if arg is a domain variable
-        or an expression of a domain variable.
-
-        See also evaluate.
-
-        """
-        from .noiseomegaexpr import noiseomegaExpr        
-
-        if id(arg) == id(self.var):
-            return self.copy()
-
-        if arg == omegasym:
-            # Convert fsym to omegasym
-            return noiseomegaExpr(self.subs(self.var, arg / (2 * pi)),
-                                  nid=self.nid, **assumptions)
-        return super(noisefExpr, self).__call__(arg, nid=self.nid, **assumptions)
-
     def plot(self, fvector=None, **kwargs):
         """Plot frequency response at values specified by fvector.
 
@@ -86,7 +71,60 @@ class noisefExpr(noiseExpr):
 
         return plot_frequency(self, fvector, **kwargs)
 
+    def transform(self, arg, **assumptions):
+        """Transform into a different domain."""
+
+        arg = expr(arg)        
+        if isinstance(arg, omegaExpr):
+            result = self.subs(omega / (2 * pi))
+            return result.subs(arg, **assumptions)            
+        elif isinstance(arg, fExpr):
+            return self.subs(arg, **assumptions)
+
+        return super(noisefExpr, self).transform(arg, **assumptions)
+    
+
+class Vfnoisy(noisefExpr):
+    """Voltage noise amplitude spectral density (units V/rtHz).
+    This can be a function of linear frequency, f.  For example,
+    to model an opamp voltage noise:
+
+    v = Vfnoisy(1e-8 / sqrt(f) + 8e-9)
+    
+    """
+
+    quantity = 'Voltage noise spectral density'
+    units = 'V/rtHz'
+
+    
+    def __init__(self, val, **assumptions):
+
+        super(Vfnoisy, self).__init__(val, **assumptions)
+        # FIXME
+        self._fourier_conjugate_class = Vt
+        self._subs_classes = {omegaExpr: Vnoisy, fExpr: Vfnoisy}
+
+
+class Ifnoisy(noisefExpr):
+    """Current noise amplitude spectral density (units A/rtHz).
+
+    This can be a function of linear frequency, f.  For example,
+    to model an opamp current noise:
+
+    i = Ifnoisy(3e-12 / sqrt(f) + 200e-15)
+    """
+
+    quantity = 'Current noise spectral density'
+    units = 'A/rtHz'
+
+    def __init__(self, val, **assumptions):
+
+        super(Ifnoisy, self).__init__(val, **assumptions)
+        # FIXME
+        self._fourier_conjugate_class = It
+        self._subs_classes = {omegaExpr: Inoisy, fExpr: Ifnoisy}
+
     
 from .texpr import It, Vt
 from .fexpr import f
-from .omegaexpr import omega
+from .noiseomegaexpr import Vnoisy, Inoisy
