@@ -15,7 +15,7 @@ currents, respectively, to model transient responses.
 One-ports can either be connected in series (+) or parallel (|) to
 create a new one-port.
 
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 """
 
 # TODO.   Rethink best way to handle impedances.   These can either be
@@ -27,18 +27,19 @@ Copyright 2014--2019 Michael Hayes, UCECE
 
 from __future__ import division
 from .functions import Heaviside, cos, exp
-from .sym import omega0sym
+from .sym import omega0sym, tsym, oo
 from .symbols import j, t, s
 from .network import Network
 from .immitance import ImmitanceMixin
 from .impedance import Impedance
 from .admittance import Admittance
+from sympy import Derivative, Integral
 
 
 __all__ = ('V', 'I', 'v', 'i', 'R', 'L', 'C', 'G', 'Y', 'Z',
            'Vdc', 'Vstep', 'Idc', 'Istep', 'Vac', 'sV', 'sI',
            'Iac', 'Vnoise', 'Inoise', 
-           'Par', 'Ser', 'Xtal', 'FerriteBead', 'CPE')
+           'Par', 'Ser', 'Xtal', 'FerriteBead', 'CPE', 'series', 'parallel', 'ladder')
 
 def _check_oneport_args(args):
 
@@ -314,6 +315,14 @@ class OnePort(Network, ImmitanceMixin):
             return self + Vn
         return self
 
+        def i_equation(self, v, kind='t'):
+            
+            raise NotImplementedError('i_equation not defined')
+
+        def v_equation(self, i, kind='t'):
+            
+            raise NotImplementedError('v_equation not defined')        
+        
     
 class ParSer(OnePort):
     """Parallel/serial class"""
@@ -417,15 +426,15 @@ class ParSer(OnePort):
                 return R(arg1._R + arg2._R)
             if isinstance(arg1, L):
                 # The currents should be the same!
-                if arg1.i0 != arg2.i0 or arg1.hasic != arg2.hasic:
+                if arg1.i0 != arg2.i0 or arg1.has_ic != arg2.has_ic:
                     raise ValueError('Series inductors with different'
                           ' initial currents!')
-                i0 = arg1.i0 if arg1.hasic else None
+                i0 = arg1.i0 if arg1.has_ic else None
                 return L(arg1.L + arg2.L, i0)
             if isinstance(arg1, G):
                 return G(arg1._G * arg2._G / (arg1._G + arg2._G))
             if isinstance(arg1, C):
-                v0 = arg1.v0 + arg2.v0 if arg1.hasic or arg2.hasic else None
+                v0 = arg1.v0 + arg2.v0 if arg1.has_ic or arg2.has_ic else None
                 return C(arg1.C * arg2.C / (arg1.C + arg2.C), v0)
             return None
 
@@ -441,15 +450,15 @@ class ParSer(OnePort):
                 return G(arg1._G + arg2._G)
             if isinstance(arg1, C):
                 # The voltages should be the same!
-                if arg1.v0 != arg2.v0 or arg1.hasic != arg2.hasic:
+                if arg1.v0 != arg2.v0 or arg1.has_ic != arg2.has_ic:
                     raise ValueError('Parallel capacitors with different'
                           ' initial voltages!')
-                v0 = arg1.v0 if arg1.hasic else None
+                v0 = arg1.v0 if arg1.has_ic else None
                 return C(arg1.C + arg2.C, v0)
             if isinstance(arg1, R):
                 return R(arg1._R * arg2._R / (arg1._R + arg2._R))
             if isinstance(arg1, L):
-                i0 = arg1.i0 + arg2.i0 if arg1.hasic or arg2.hasic else None
+                i0 = arg1.i0 + arg2.i0 if arg1.has_ic or arg2.has_ic else None
                 return L(arg1.L * arg2.L / (arg1.L + arg2.L), i0)
             return None
 
@@ -561,6 +570,9 @@ class Par(ParSer):
 
     def __init__(self, *args):
 
+        if len(args) < 2:
+            raise ValueError('Par requires at least two args')
+        
         _check_oneport_args(args)
         super(Par, self).__init__()
         self.args = args
@@ -595,12 +607,12 @@ class Par(ParSer):
             total += arg.height
         return total + (len(self.args) - 1) * self.hsep
 
-    def net_make(self, net, n1=None, n2=None):
+    def _net_make(self, netlist, n1=None, n2=None, dir='right'):
 
         s = []
         if n1 is None:
-            n1 = net.node
-        n3, n4 =  net.node, net.node
+            n1 = netlist._node
+        n3, n4 =  netlist._node, netlist._node
 
         H = [(arg.height + self.hsep) * 0.5 for arg in self.args]
         
@@ -609,11 +621,16 @@ class Par(ParSer):
 
         # Draw component in centre if have odd number in parallel.
         if (N & 1):
-            s.append(self.args[N // 2].net_make(net, n3, n4))
+            s.append(self.args[N // 2]._net_make(netlist, n3, n4, dir))
 
         na, nb = n3, n4
 
-        s.append('W %s %s; right=%s' % (n1, n3, self.wsep))
+        s.append('W %s %s; %s=%s' % (n1, n3, dir, self.wsep))
+
+        if dir == 'right':
+            updir, downdir = 'up', 'down'
+        else:
+            updir, downdir = 'right', 'left'
 
         # Draw components above centre
         for n in range(num_branches):
@@ -623,10 +640,10 @@ class Par(ParSer):
             else:
                 sep = H[N // 2 - n] + H[N // 2 - 1 - n]
 
-            nc, nd =  net.node, net.node
-            s.append('W %s %s; up=%s' % (na, nc, sep))
-            s.append('W %s %s; up=%s' % (nb, nd, sep))
-            s.append(self.args[N // 2 - 1 - n].net_make(net, nc, nd))
+            nc, nd =  netlist._node, netlist._node
+            s.append('W %s %s; %s=%s' % (na, nc, updir, sep))
+            s.append('W %s %s; %s=%s' % (nb, nd, updir, sep))
+            s.append(self.args[N // 2 - 1 - n]._net_make(netlist, nc, nd, dir))
             na, nb = nc, nd
 
         na, nb = n3, n4
@@ -639,16 +656,16 @@ class Par(ParSer):
             else:
                 sep = H[(N + 1) // 2 + n] + H[(N + 1) // 2 - 1 + n]
 
-            nc, nd =  net.node, net.node
-            s.append('W %s %s; down=%s' % (na, nc, sep))
-            s.append('W %s %s; down=%s' % (nb, nd, sep))
-            s.append(self.args[(N + 1) // 2 + n].net_make(net, nc, nd))
+            nc, nd =  netlist._node, netlist._node
+            s.append('W %s %s; %s=%s' % (na, nc, downdir, sep))
+            s.append('W %s %s; %s=%s' % (nb, nd, downdir, sep))
+            s.append(self.args[(N + 1) // 2 + n]._net_make(netlist, nc, nd, dir))
             na, nb = nc, nd
 
         if n2 is None:
-            n2 = net.node
+            n2 = netlist._node
 
-        s.append('W %s %s; right=%s' % (n4, n2, self.wsep))
+        s.append('W %s %s; %s=%s' % (n4, n2, dir, self.wsep))
         return '\n'.join(s)
 
     @property
@@ -669,6 +686,9 @@ class Ser(ParSer):
 
     def __init__(self, *args):
 
+        if len(args) < 2:
+            raise ValueError('Ser requires at least two args')
+        
         _check_oneport_args(args)
         super(Ser, self).__init__()
         self.args = args
@@ -702,20 +722,20 @@ class Ser(ParSer):
             total += arg.width
         return total + (len(self.args) - 1) * self.wsep
 
-    def net_make(self, net, n1=None, n2=None):
+    def _net_make(self, netlist, n1=None, n2=None, dir='right'):
 
         s = []
         if n1 is None:
-            n1 = net.node
+            n1 = netlist._node
         for arg in self.args[:-1]:
-            n3 = net.node
-            s.append(arg.net_make(net, n1, n3))
-            n1 = net.node
-            s.append('W %s %s; right=%s' % (n3, n1, self.wsep))
+            n3 = netlist._node
+            s.append(arg._net_make(netlist, n1, n3, dir))
+            n1 = netlist._node
+            s.append('W %s %s; %s=%s' % (n3, n1, dir, self.wsep))
 
         if n2 is None:
-            n2 = net.node
-        s.append(self.args[-1].net_make(net, n1, n2))
+            n2 = netlist._node
+        s.append(self.args[-1]._net_make(netlist, n1, n2, dir))
         return '\n'.join(s)
 
     @property
@@ -733,43 +753,65 @@ class Ser(ParSer):
 class R(OnePort):
     """Resistor"""
 
+    resistor = True    
+    
     def __init__(self, Rval):
 
         self.args = (Rval, )
         self._R = cExpr(Rval)
         self._Z = Impedance(self._R)
 
+    def i_equation(self, v, kind='t'):
+
+        return Current(Voltage(v).select(kind) / self._Z.selectexpr(kind)).select(kind)
+
+    def v_equation(self, i, kind='t'):
+
+        return Voltage(Current(i).select(kind) * self._Z.selectexpr(kind)).select(kind)
+    
 
 class G(OnePort):
     """Conductance"""
 
+    resistor = True    
+
     def __init__(self, Gval):
 
         self.args = (Gval, )
         self._G = cExpr(Gval)
-        self._Z = 1 / Admittance(self._G)
+        self._Z = Impedance(1 / self._G)
 
-    def net_make(self, net, n1=None, n2=None):
+    def _net_make(self, netlist, n1=None, n2=None, dir='right'):
 
         if n1 == None:
-            n1 = net.node
+            n1 = netlist._node
         if n2 == None:
-            n2 = net.node
-        return 'R? %s %s {%s}; right' % (n1, n2, 1 / self._G)
+            n2 = netlist._node
+        return 'R? %s %s {%s}; %s' % (n1, n2, 1 / self._G, dir)
 
+    def i_equation(self, v, kind='t'):
+
+        return Current(Voltage(v).select(kind) / self._Z.selectexpr(kind)).select(kind)
+
+    def v_equation(self, i, kind='t'):
+
+        return Voltage(Current(i).select(kind) * self._Z.selectexpr(kind)).select(kind)
+    
 
 class L(OnePort):
     """Inductor
 
     Inductance Lval, initial current i0"""
 
+    inductor = True
+
     def __init__(self, Lval, i0=None):
 
-        self.hasic = i0 is not None
+        self.has_ic = i0 is not None
         if i0 is None:
             i0 = 0
 
-        if self.hasic:
+        if self.has_ic:
             self.args = (Lval, i0)
         else:
             self.args = (Lval, )
@@ -782,19 +824,37 @@ class L(OnePort):
         self._Voc = Voltage(-Vs(i0 * Lval))
         self.zeroic = self.i0 == 0 
 
+    def i_equation(self, v, kind='t'):
 
+        from .sym import tausym
+
+        if kind in ('t', 'time', 'super'):
+            u = tausym
+            v = expr(v).subs(t, u)
+            return Current(expr(Integral(v.expr, (u, -oo, tsym))) / self.L).select(kind)
+        return Current(Voltage(v).select(kind) / self.Z.selectexpr(kind)).select(kind)
+
+    def v_equation(self, i, kind='t'):
+
+        if kind in ('t', 'time', 'super'):
+            return Voltage(self.L * expr(Derivative(i.expr, t))).select(kind)
+        return Voltage(Current(i).select(kind) * self._Z.selectexpr(kind)).select(kind)
+
+    
 class C(OnePort):
     """Capacitor
 
     Capacitance Cval, initial voltage v0"""
 
+    capacitor = True    
+
     def __init__(self, Cval, v0=None):
 
-        self.hasic = v0 is not None
+        self.has_ic = v0 is not None
         if v0 is None:
             v0 = 0
 
-        if self.hasic:
+        if self.has_ic:
             self.args = (Cval, v0)
         else:
             self.args = (Cval, )
@@ -807,6 +867,22 @@ class C(OnePort):
         self._Voc = Voltage(Vs(v0) / s)
         self.zeroic = self.v0 == 0
 
+    def i_equation(self, v, kind='t'):
+        
+        if kind in ('t', 'time', 'super'):
+            return Current(self.C * expr(Derivative(v.expr, t))).select(kind)
+        return Current(Voltage(v).select(kind) / self._Z.selectexpr(kind)).select(kind)
+
+    def v_equation(self, i, kind='t'):
+
+        from .sym import tausym
+
+        if kind in ('t', 'time', 'super'):
+            u = tausym
+            i = expr(i).subs(t, u)
+            return Voltage(expr(Integral(i.expr, (u, -oo, tsym))) / self.C).select(kind)
+        return Voltage(Current(i).select(kind) * self._Z.selectexpr(kind)).select(kind)
+
 
 class CPE(OnePort):
     """Constant phase element
@@ -860,6 +936,10 @@ class VoltageSource(OnePort):
     netname = 'V'
     is_noisy = False
 
+    def v_equation(self, i, kind='t'):
+        
+        return Voltage(self.voc).select(kind)
+    
 
 class sV(VoltageSource):
     """Arbitrary s-domain voltage source"""
@@ -986,6 +1066,10 @@ class CurrentSource(OnePort):
         """Open-circuit current of a current source.  To achieve this the
         open-circuit voltage needs to be infinite."""
         return self.Isc
+
+    def i_equation(self, v, kind='t'):
+
+        return Current(self.isc).select(kind)        
     
     
 class sI(CurrentSource):
@@ -1122,11 +1206,11 @@ class Xtal(OnePort):
 
         return (R(self.R1) + L(self.L1) + C(self.C1)) | C(self.C0)
 
-    def net_make(self, net, n1=None, n2=None):
+    def _net_make(self, netlist, n1=None, n2=None, dir='right'):
 
         # TODO: draw this with a symbol
         net = self.expand()
-        return net.net_make(net, n1, n2)    
+        return net._net_make(netlist, n1, n2, dir)    
 
 
 class FerriteBead(OnePort):
@@ -1150,11 +1234,11 @@ class FerriteBead(OnePort):
 
         return R(self.Rs) + (R(self.Rp) + L(self.Lp) + C(self.Cp))
 
-    def net_make(self, net, n1=None, n2=None):
+    def _net_make(self, netlist, n1=None, n2=None, dir='right'):
 
         # TODO: draw this with a symbol
         net = self.expand()
-        return net.net_make(net, n1, n2)            
+        return net._net_make(netlist, n1, n2, dir)            
     
 class LoadCircuit(Network):
     """Circuit comprised of a load oneport connected in parallel with a
@@ -1189,10 +1273,10 @@ class LoadCircuit(Network):
         """Time-domain current into load."""
         return self.inet.isc
 
-    def net_make(self, net, n1=None, n2=None):
+    def _net_make(self, netlist, n1=None, n2=None, dir='right'):
 
         # TODO: draw this better rather than as a oneport.
-        return self.vnet.net_make(self.vnet, n1, n2)
+        return self.vnet._net_make(netlist, n1, n2, dir)
 
 
 class ControlledSource(OnePort):
@@ -1295,9 +1379,56 @@ class Damper(R):
     Friction coeff rval"""
     pass
 
+
+def series(*args):
+    """Create a series combination of a number of components.
+    Args that are None are ignored.  If there is only one
+    non-None component, return that component."""
+
+    args = [net for net in args if net is not None]
+    if args == []:
+        return None
+    if len(args) == 1:
+        return args[0]
+    return Ser(*args)
+
+
+def parallel(*args):
+    """Create a parallel combination of a number of components.
+    Args that are None are ignored.  If there is only one
+    non-None component, return that component."""    
+
+    args = [net for net in args if net is not None]
+    if args == []:
+        return None
+    if len(args) == 1:
+        return args[0]
+    return Par(*args)
+
+
+def ladder(*args, **kwargs):
+    """Create a ladder oneport network with alternating series and shunt components.
+    If an arg is None, the component is ignored.
+
+    ladder(R(1), C(2), R(3)) is equivalent to R(1) + (C(1) | R(3))
+
+    ladder(None, R(1), C(2), R(3)) is equivalent to R(1) | (C(1) + R(3))
+    """
+
+    start_series = kwargs.pop('start_series', True)
+    
+    if len(args) == 0:
+        return None
+    elif len(args) == 1:
+        return args[0]
+    elif start_series:
+        return series(args[0], ladder(*args[1:], start_series = not start_series))
+    else:
+        return parallel(args[0], ladder(*args[1:], start_series = not start_series))
+
     
 # Imports at end to circumvent circular dependencies
-from .expr import Expr
+from .expr import Expr, expr
 from .cexpr import cExpr, Iconst, Vconst
 from .sexpr import sExpr, Is, Vs, Ys, Zs
 from .texpr import tExpr
