@@ -1,4 +1,4 @@
-"""This file provides the State class that maintains the global state.
+"""This module provides the State class that maintains the global state.
 
 Copyright 2014--2020 Michael Hayes, UCECE
 
