@@ -1,11 +1,11 @@
 """
 This module performs state-space analysis.
 
-Copyright 2019 Michael Hayes, UCECE
+Copyright 2019-2020 Michael Hayes, UCECE
 
 """
 
-from .mnacpts import L, C, I, V
+from .mnacpts import I, V, C, L
 from .matrix import Matrix
 from .smatrix import sMatrix
 from .tmatrix import tMatrix
@@ -42,7 +42,10 @@ def _hack_vars(exprs):
                 exprs[m] = expr
 
 class StateSpace(object):
-    """This converts a circuit to state-space representation."""
+    """This converts a circuit to state-space representation.
+
+    It can take a long time for a symbolic circuit with many reactive
+    components."""
 
     def __init__(self, cct, node_voltages=True, branch_currents=False):
 
@@ -65,12 +68,12 @@ class StateSpace(object):
             name = elt.name
             cpt_map[name] = sselt.name
             
-            if isinstance(elt, L):
+            if elt.is_inductor:
                 if sselt.name in cct.elements:
                     raise ValueError('Name conflict %s, either rename the component or improve the code!' % sselt.name)
 
                 inductors.append(elt)
-            elif isinstance(elt, C):
+            elif elt.is_capacitor:
                 if sselt.name in cct.elements:
                     raise ValueError('Name conflict %s, either rename the component or improve the code!' % sselt.name)
                 
@@ -81,7 +84,8 @@ class StateSpace(object):
         self.cct = cct
         self.sscct = sscct
         # sscct can be analysed in the time domain since it has no
-        # reactive components.
+        # reactive components.  However, for large circuits
+        # this can take a long time due to inversion of the MNA matrix.
         
         dotx_exprs = []
         statevars = []
