@@ -1,3 +1,11 @@
+"""This module provides the Super class.  It is the base class for
+Current and Voltage.  It represents voltages and currents as a
+superposition in different transform domains.
+
+Copyright 2019--2020 Michael Hayes, UCECE
+
+"""
+
 from __future__ import division
 from .expr import Expr, ExprDict, expr
 from .sym import tsym, omega0sym, symbols_find, is_sympy, symsymbol
@@ -50,9 +58,8 @@ class Super(ExprDict):
     def __init__(self, *args, **kwargs):
         super (Super, self).__init__()
 
-        if any(args):
-            for arg in args:
-                self.add(arg)
+        for arg in args:
+            self.add(arg)
 
     def _representation(self):
         if not any(self):
@@ -152,22 +159,22 @@ class Super(ExprDict):
         return self.ac_keys() != []
 
     @property
-    def has_s(self):
+    def has_s_transient(self):
         """True if have transient component defined in the s-domain."""
         return 's' in self
 
     @property
-    def has_t(self):
+    def has_t_transient(self):
         """True if have transient component defined in the time-domain."""
         return 't' in self
 
     @property
     def has_transient(self):
         """True if have transient component."""        
-        return self.has_s or self.has_t
+        return self.has_s_transient or self.has_t_transient
 
     @property
-    def has_n(self):
+    def has_noisy(self):
         """True if there is a noise component."""                
         return self.noise_keys() != []
 
@@ -182,9 +189,9 @@ class Super(ExprDict):
         return self.has_ac and self.ac_keys() == list(self.keys())
 
     @property
-    def is_n(self):
+    def is_noisy(self):
         """True if only has noise components."""                                
-        return self.has_n and self.noise_keys() == list(self.keys())
+        return self.has_noisy and self.noise_keys() == list(self.keys())
 
     @property
     def is_s_transient(self):
@@ -236,6 +243,29 @@ class Super(ExprDict):
             new[kind] = value.subs(*args, **kwargs)
         return new        
     
+    def initial_value(self):
+        """Determine value at t = 0. 
+        See also pre_initial_value and post_initial_value"""
+
+        return self.time().initial_value()
+
+    def pre_initial_value(self):
+        """Determine value at t = 0-.
+        See also initial_value and post_initial_value"""
+
+        return self.time().pre_initial_value()
+
+    def post_initial_value(self):
+        """Determine value at t = 0+.
+        See also pre_initial_value and initial_value"""
+
+        return self.time().post_initial_value()
+
+    def final_value(self):
+        """Determine value at t = oo."""
+
+        return self.time().final_value()
+
     def transform(self, arg, **assumptions):
         """Transform into a different domain."""        
 
@@ -471,8 +501,13 @@ class Super(ExprDict):
         """Add a value into the superposition."""
 
         # Avoid triggering __eq__ for Super otherwise have infinite recursion
-        if not isinstance(value, Super) and value == 0:
-            return
+        if not isinstance(value, Super):
+            try:
+                val = value.expr
+            except:
+                val = value
+            if val == 0:
+                return
 
         if '_decomposition' in self:
             delattr(self, '_decomposition')
