@@ -198,6 +198,11 @@ def sympify(expr, evaluate=True, **assumptions):
         assumptions['positive'] = True
         # Note this implies that imag is False.   Also note that all
         # reals are considered complex (but with a zero imag part).
+
+    elif 'positive' in assumptions:
+        if not assumptions['positive']:
+            assumptions.pop('positive')
+        
     return sympify1(expr, state.context.symbols, evaluate, **assumptions)
 
 
@@ -324,6 +329,7 @@ pi = sym.pi
 j = sym.I
 oo = sym.oo
 inf = sym.oo
+one = sym.S.One
 
 try:
     from sympy.core.function import AppliedUndef
